# -*- coding: utf-8 -*-

# Handles writing and loading of configuration files

"""
Copyright (C) 2006-2009 ianaré sévi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

ianare@gmail.com
"""

import wx
import codecs
import time
import re
from xml.dom import minidom

import utils
import operations



#------- Save Configuration File -------#

#- Get correct values depending on child type
def getChildValues(child):
    value = None
    if isinstance(child, wx.CheckBox) or isinstance(child, wx.RadioButton):
        value = int(child.GetValue())
    elif isinstance(child, wx.TextCtrl) or isinstance(child, wx.SpinCtrl):
        value = child.GetValue()
    elif isinstance(child, wx.ComboBox):
        value = child.GetStringSelection()
    elif isinstance(child, wx.Choice):
        value = child.GetSelection()

    if value is not None:
        type = child.GetClassName()
        id = child.GetName()
        return id,type,value

#- Get complete path down hierachy to value
#- End result is tree structure of values.
def getAllChildParams(cfg, child, r):
    # panels & notebook get children
    if isinstance(child, wx.Notebook) or isinstance(child, wx.Panel):
        type = child.GetClassName()
        id = child.GetName()
        parent = child.GetParent().GetName()
        cfg += '\t\t\t'+r*'\t'+u'<%s id="%s">\n'%(type,id)
        for child in child.GetChildren():
            try:
                cfg = getAllChildParams(cfg, child, r+1)
            except TypeError:
                pass
        cfg += '\t\t\t'+r*'\t'+u"</%s>\n"%type
        return cfg
    # others get values
    else:
        try:
            id,type,value = getChildValues(child)
        except TypeError:
            pass
        else:
            cfg += '\t\t\t'+r*'\t'+u'<value id="%s" type="%s">%s</value>\n'\
                  %(id,type,value)
        return cfg

#- Get values, write configuration file
def saveConfig(main):
    datetime = utils.uDate(main, '%Y-%m-%d %H:%M:%S', time.localtime())
    cfgFile = u'<?xml version="1.0" encoding="UTF-8"?>\n'
    cfgFile += u'<configuration application="Métamorphose-2" '+\
               'version="%s" datetime="%s">\n'\
               %(main.version, datetime)
    # get info for 'normal' notebook tabs
    pages = utils.getNotebookPages(main)
    for i in (0,2,3):
        # section header is page name
        cfgFile += u'\t<page id="%s" name="%s">\n'%(i,pages[i].GetName())

        # XXX needs work - doesn't do tree, use getAllChildParams?
        for child in pages[i].GetChildren():
            try:
                id,type,value = getChildValues(child)
            except TypeError:
                pass
            else:
                cfgFile += u'\t\t<value id="%s" type="%s">%s</value>\n'\
                           %(id,type,value)
        cfgFile += u'\t</page>\n'

    # get renaming operations
    cfgFile += u'\t<page id="1" name="%s">\n'%pages[1].GetName()
    operationList = main.renamer.Panel.usedOperations
    operationStack = main.renamer.operations

    for i in range(operationList.GetItemCount()):
        type = operationList.GetItemText(i)
        type = re.sub("\d{1,}: ", '', type)
        type = operations.defs[type][0]

        cfgFile += u'\t\t<operation id="%s" type="%s">\n'%(i, type)

        for child in operationStack[i].GetChildren():
            cfgFile = getAllChildParams(cfgFile, child, 0)

        cfgFile += u'\t\t</operation>\n'
    cfgFile += "\t</page>\n</configuration>\n"

    # save dialog
    dlg = wx.FileDialog(None, message=_(u"Save configuration as ..."),
          defaultDir=utils.realPath(u'configs'), defaultFile=u'',
          wildcard=_(u"Configuration file (*.cfg)")+u'|*.cfg',
          style=wx.SAVE|wx.OVERWRITE_PROMPT
          )
    # write file
    if dlg.ShowModal() == wx.ID_OK:
        utils.writeFile(dlg.GetPath(), cfgFile)
    dlg.Destroy()



#------ Load Configuration File --------#

#- Set widgets values correctly depending on type
def setWidgetValues(id, type, value):
    if value == None: value = ''

    if type == 'wxCheckBox' or type == 'wxRadioButton' or\
     type == 'wxSpinCtrl':
        id.SetValue(int(value))
    elif type == 'wxTextCtrl':
        id.SetValue(value)
    elif type == 'wxComboBox':
        id.SetStringSelection(value)
    elif type == 'wxChoice':
        id.SetSelection(int(value))


# Get widget values recursively through XML tree,
# apply setings to operation
def applyValues(parent, node):
    id = node.attributes['id'].value
    # value, apply
    if node.tagName == 'value':
        wtype = node.attributes['type'].value
        try:
            value = node.childNodes[0].nodeValue
        except IndexError:
            value = None
        try:
            widget = getattr(parent, id)
        except AttributeError:
            pass
        else:
            setWidgetValues(widget, wtype, value)
    # notebook, cycle through pages
    elif node.tagName == 'wxNotebook':
        notebook = getattr(parent, id)
        names = []
        for i in range(notebook.GetPageCount()):
            names.append(notebook.GetPage(i).GetName())
        for child in node.getElementsByTagName('wxPanel'):
            childname = child.attributes['id'].value
            if childname in names:
                applyValues(parent, child)
    # panel, cycle through panels and values
    elif node.tagName == 'wxPanel':
        #print parent, id
        parent = getattr(parent, id)
        for child in node.childNodes:
            if child.nodeType == 1:
                applyValues(parent, child)


#- Read file and apply settings
def loadConfig(main, configFilePath):
    # open config file
    if main.debug:
        print "opening config : %s" % configFilePath
    try:
        xmldoc = codecs.open(configFilePath,'r', 'utf-8')
    except:
        print "Invalid config file : %s" % configFilePath
    else:
        xmldoc = minidom.parseString(xmldoc.read().encode("utf-8"))
        config = xmldoc.firstChild
        pages = utils.getNotebookPages(main)

        # set autopreview to false while loading config
        # avoids loading paths more than once
        v = main.bottomWindow.autoPreview.GetValue()
        main.bottomWindow.autoPreview.SetValue(False)
        configVersion = config.attributes['version'].value

        if main.debug:
            print "config version : %s" % configVersion

        for page in config.getElementsByTagName('page'):
            page_name = page.attributes['name'].value
            page_id = int(page.attributes['id'].value)
            # normal pages
            if page_name != 'mainPanel':
                for node in page.getElementsByTagName('value'):
                    id = node.attributes['id'].value
                    # make sure element exists before setting it
                    if hasattr(pages[page_id], id):
                        id = getattr(pages[page_id], id)
                    else:
                        continue
                    type = node.attributes['type'].value
                    try:
                        value = node.childNodes[0].nodeValue
                    except IndexError:
                        value = None
                    setWidgetValues(id, type, value)
                # adjustments if needed
                if hasattr(pages[page_id], 'onConfigLoad'):
                    getattr(pages[page_id], 'onConfigLoad')()
            # main page
            else:
                # cleanup before loading
                pages[1].destroyAllOperations()

                # load values
                for op in page.getElementsByTagName('operation'):
                    type = op.attributes['type'].value
                    type = operations.getTranslation(type)
                    # make sure we have a valid type
                    if type == False:
                        continue
                    id = op.attributes['id'].value
                    pages[1].addOperationToStack(type, id)

                    # apply node values to widgets
                    operation = main.renamer.operations[int(id)]
                    for node in op.childNodes:
                        if node.nodeType == 1:
                            applyValues(operation, node)

                    # apply each operation's post load routine
                    if hasattr(operation, 'onConfigLoad'):
                        operation.onConfigLoad()

        # preview
        main.bottomWindow.autoPreview.SetValue(v)
        if main.autoModeLevel != 0 or\
         (main.prefs['previewOnConfig'] and main.autoModeLevel is False):
            picker = main.picker.Panel
            if main.prefs['useDirTree'] and \
             picker.dirPicker.GetPath() != picker.path.GetValue():
                picker.dirPicker.unbind()
                picker.dirPicker.SetPath(picker.path.GetValue())
                picker.dirPicker.bind()
            main.picker.refresh(True)
            
            
#- Get config file path from dialog
def loadConfigDialog(main):
    dlg = wx.FileDialog(None,
      message=_(u"Load a configuration file"),
      defaultDir=utils.realPath(u'configs'), defaultFile=u'',
      wildcard=_(u"Configuration file (*.cfg)")+u'|*.cfg',
      style=wx.OPEN
      )
    if dlg.ShowModal() == wx.ID_OK:
        loadConfig(main, dlg.GetPath())
    dlg.Destroy()
